@@ -10,6 +10,8 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <math.h>
+
 #include <X11/Xlib.h>
 
 #include "tcltk/tclmagic.h"
@@ -22,6 +24,7 @@
 #include "utils/signals.h"
 #include "utils/utils.h"
 #include "utils/hash.h"
+#include "utils/malloc.h"
 #include "graphics/grTkInt.h"
 #include "graphics/grTkCommon.h"
 #include "database/fonts.h"
@@ -305,107 +308,120 @@ grtkRectConvert(mr, xr)
  */
 
 void
-grtkFontText(text, font, size, rotate, offset, just, pos, clip, obscure, bboxret)
+grtkFontText(text, font, size, rotate, pos, clip, obscure)
     char *text;
     int font;
-    int size;
-    int rotate;
-    Point *offset;
-    int just;
-    Point *pos;
+    int size;			/* pixel size of the text */
+    int rotate;			/* text rotation */
+    Point *pos;			/* text base position */
     Rect *clip;
     LinkedRect *obscure;
-    Rect *bboxret;
 {
     char *tptr;
     FontChar *ccur, *clist;
-    Point *coffset, *tp, loffset;
-    Rect *cbbox, locbbox;
-    int np, i, w, h, llx, lly;
+    Point *coffset, *tp, loffset, locoffset, corners[4], lpos;
+    Rect *cbbox, charbbox, *frect;
+    int np, i, j, w, h, llx, lly, baseline;
     XPoint *xp;
     Pixmap pxm;
+    double fscale, scx, scy, tmpx, tmpy, rrad, cr, sr;
     static GC fontgc = (GC)NULL;
 
-    if (font < 0)
-    {
-	GrTextSize(text, GR_TEXT_MEDIUM, bboxret);
-	grtkPutText(text, pos, clip, obscure);
-	return;
-    }
+    frect = &DBFontList[font]->mf_extents;
+    fscale = (double)size / (double)frect->r_ytop;
+    rrad = (double)rotate * 0.0174532925;
+    cr = cos(rrad);
+    sr = sin(rrad);
+    lpos = GeoOrigin;
 
-    /* Create a bitmap large enough to hold any rotated character */
+    /* 1st pass: find the descent of the string */
 
-    llx = DBFontList[font]->mf_extents.r_xbot;
-    lly = DBFontList[font]->mf_extents.r_ybot;
-    w = ((DBFontList[font]->mf_extents.r_xtop - llx) * size) / 1024;
-    h = ((DBFontList[font]->mf_extents.r_ytop - lly) * size) / 1024;
-
-    pxm = XCreatePixmap(grXdpy, grCurrent.windowid, w, h, 1);
-    if (fontgc == (GC)NULL)
-    {
-	XGCValues values;
-	values.foreground = 0;
-	values.background = 0;
-	fontgc = XCreateGC(grXdpy, pxm, GCForeground | GCBackground, &values);
-    }
-
-    /* Parse label once to get the bounding box */
-    locbbox = GeoNullRect;
+    baseline = 0;
     for (tptr = text; *tptr != '\0'; tptr++)
     {
-        DBFontChar(font, *tptr, NULL, &coffset, &cbbox);
-        if (*(tptr + 1) == '\0')
-            locbbox.r_xtop += cbbox->r_xtop;
-        else
-            locbbox.r_xtop += coffset->p_x;
-        locbbox.r_ytop = MAX(locbbox.r_ytop, cbbox->r_ytop);
-        locbbox.r_ybot = MIN(locbbox.r_ybot, cbbox->r_ybot);
-    }
-    locbbox.r_xbot = (locbbox.r_xbot * size) / 1024;
-    locbbox.r_ybot = (locbbox.r_ybot * size) / 1024;
-    locbbox.r_xtop = (locbbox.r_xtop * size) / 1024;
-    locbbox.r_ytop = (locbbox.r_ytop * size) / 1024;
-
-    /* Compute lower left point based on position and justification */
-
-    loffset.p_x = pos->p_x;
-    loffset.p_y = pos->p_y;
-
-    switch(just)		/* justification */
-    {
-	case GEO_NORTH:
-	    loffset.p_x -= (locbbox.r_xtop >> 1);
-	    break;
-	case GEO_NORTHWEST:
-	    loffset.p_x -= locbbox.r_xtop;
-	    break;
-	case GEO_WEST:
-	    loffset.p_x -= locbbox.r_xtop;
-	    loffset.p_y -= (locbbox.r_ytop >> 1);
-	    break;
-	case GEO_SOUTHWEST:
-	    loffset.p_x -= locbbox.r_xtop;
-	    loffset.p_y -= locbbox.r_ytop;
-	    break;
-	case GEO_SOUTH:
-	    loffset.p_x -= (locbbox.r_xtop >> 1);
-	    loffset.p_y -= locbbox.r_ytop;
-	    break;
-	case GEO_SOUTHEAST:
-	    loffset.p_y -= locbbox.r_ytop;
-	    break;
-	case GEO_EAST:
-	    loffset.p_y -= (locbbox.r_ytop >> 1);
-	    break;
-	case GEO_CENTER:
-	    loffset.p_x -= (locbbox.r_xtop >> 1);
-	    loffset.p_y -= (locbbox.r_ytop >> 1);
-	    break;
+        DBFontChar(font, *tptr, NULL, NULL, &cbbox);
+	if (cbbox->r_ybot < -baseline)
+	    baseline = -cbbox->r_ybot;
     }
+    baseline = (int)((double)baseline * fscale);
 
     for (tptr = text; *tptr != '\0'; tptr++)
     {
-	DBFontChar(font, *tptr, &clist, &coffset, &cbbox);
+	scx = (double)lpos.p_x * fscale;
+	scy = (double)lpos.p_y * fscale;
+
+	tmpx = scx * cr + scy * sr;
+	tmpy = scy * cr - scx * sr;
+
+	loffset.p_x = pos->p_x + (int)round(tmpx);
+	loffset.p_y = grMagicToX(pos->p_y + baseline) + (int)round(tmpy);
+
+        DBFontChar(font, *tptr, &clist, &coffset, &cbbox);
+	np = 0;
+	for (ccur = clist; ccur != NULL; ccur = ccur->fc_next)
+	    np += ccur->fc_numpoints;
+
+	xp = (XPoint *)mallocMagic(np * sizeof(XPoint));
+
+	j = 0;
+	for (ccur = clist; ccur != NULL; ccur = ccur->fc_next)
+	{
+	    tp = ccur->fc_points;
+	    for (i = 0; i < ccur->fc_numpoints; i++, j++)
+	    {
+		scx = (double)tp[i].p_x * fscale;
+		scy = (double)tp[i].p_y * fscale;
+
+		tmpx = scx * cr - scy * sr;
+		tmpy = scx * sr + scy * cr;
+
+		xp[j].x = (int)round(tmpx);
+		xp[j].y = (int)round(tmpy);
+
+		/* Initialize bounding box */
+		if (j == 0)
+		{
+		    charbbox.r_xbot = charbbox.r_xtop = xp[j].x;
+		    charbbox.r_ybot = charbbox.r_ytop = xp[j].y;
+		}
+		else
+		{
+		    if (xp[j].x < charbbox.r_xbot)
+			charbbox.r_xbot = xp[j].x;
+		    else if (xp[j].x > charbbox.r_xtop)
+			charbbox.r_xtop = xp[j].x;
+		    if (xp[j].y < charbbox.r_ybot)
+			charbbox.r_ybot = xp[j].y;
+		    else if (xp[j].y > charbbox.r_ytop)
+			charbbox.r_ytop = xp[j].y;
+		}
+	    }
+	}
+
+	/* Create a bitmap */
+
+	w = charbbox.r_xtop - charbbox.r_xbot + 1;
+	h = charbbox.r_ytop - charbbox.r_ybot + 1;
+
+	/* Adjust all points to the bounding box origin, and invert Y */
+	for (j = 0; j < np; j++)
+	{
+	    xp[j].x -= charbbox.r_xbot;
+	    xp[j].y = charbbox.r_ytop - xp[j].y;
+	}
+
+	pxm = XCreatePixmap(grXdpy, grCurrent.windowid, w, h, 1);
+
+	if (fontgc == (GC)NULL)
+	{
+	    XGCValues values;
+	    values.foreground = 0;
+	    values.background = 0;
+	    fontgc = XCreateGC(grXdpy, pxm, GCForeground | GCBackground, &values);
+	}
+
+	locoffset.p_x = loffset.p_x + charbbox.r_xbot;
+	locoffset.p_y = loffset.p_y - charbbox.r_ytop;
 
 	XSetForeground(grXdpy, fontgc, 0);
 	XSetFunction(grXdpy, fontgc, GXcopy);
@@ -413,30 +429,25 @@ grtkFontText(text, font, size, rotate, offset, just, pos, clip, obscure, bboxret
 	XSetFunction(grXdpy, fontgc, GXxor);
 	XSetForeground(grXdpy, fontgc, 1);
 
+	j = 0;
 	for (ccur = clist; ccur != NULL; ccur = ccur->fc_next)
 	{
 	    np = ccur->fc_numpoints;
-	    tp = ccur->fc_points;
-	    xp = (XPoint *)mallocMagic(np * sizeof(XPoint));
-	    for (i = 0; i < ccur->fc_numpoints; i++)
-	    {
-		xp[i].x = ((tp[i].p_x - llx) * size) / 1024;
-		xp[i].y = h - (((tp[i].p_y - lly) * size) / 1024);
-	    }
-	    XFillPolygon(grXdpy, pxm, fontgc, xp, np, Complex, CoordModeOrigin);
+	    XFillPolygon(grXdpy, pxm, fontgc, &xp[j], np, Complex, CoordModeOrigin);
+	    j += np;
 	}
 	freeMagic((char *)xp);
 
 	XSetClipMask(grXdpy, grGCText, pxm);
-	XSetClipOrigin(grXdpy, grGCText, loffset.p_x, loffset.p_y);
-	XFillRectangle(grXdpy, grCurrent.windowid, grGCText, loffset.p_x,
-		loffset.p_y, w, h);
+	XSetClipOrigin(grXdpy, grGCText, locoffset.p_x, locoffset.p_y);
+	XFillRectangle(grXdpy, grCurrent.windowid, grGCText, locoffset.p_x,
+		locoffset.p_y, w, h);
 
-	loffset.p_x += (coffset->p_x * size) / 1024;
-	loffset.p_y += (coffset->p_y * size) / 1024;
+	lpos.p_x += coffset->p_x;
+	lpos.p_y += coffset->p_y;
+
+        XFreePixmap(grXdpy, pxm);
     }
- 
-    XFreePixmap(grXdpy, pxm);
 }
 
 /*
